/*
 * "rastertofp2200pos.c v102 2009-11-12 18:03:00"
 *
 *   Fujitsu Isotec FIT-POS filter for the Common UNIX Printing System
 *   (CUPS).
 *
 *   Copyright 2008-2009 by Fujitsu Isotec Limited.
 *
 *   These coded instructions, statements, and computer programs are the
 *   property of Apple Inc. and are protected by Federal copyright
 *   law.  Distribution and use rights are outlined in the file "LICENSE.txt"
 *   which should have been included with this file.  If this file is
 *   file is missing or damaged, see the license at "http://www.cups.org/".
 *
 *   This file is subject to the Apple OS-Developed Software exception.
 *
 * Contents:
 *
 *   GetChoiceID()      - Get PPD choice ID.
 *   OutputPrintData()  - Output print data.
 *   OutputFeed()       - Output paper feed.
 *   OutputRaster()     - Output raster data.
 *   StartJob()         - Start s job.
 *   StartPage()        - Start a page of graphics.
 *   EndPage()          - Finish a page of graphics.
 *   EndJob()           - Finish a job.
 *   CancelJob()        - Cancel the current job.
 *   main()             - Main entry and processing of driver.
 */

/*
 * Include necessary headers...
 */

#include <cups/cups.h>
#include <cups/ppd.h>
/* #include <cups/string.h> */
/* #include <cups/i18n.h> */
#include <cups/raster.h>
#include <stdlib.h>
/* #include <unistd.h> */
#include <fcntl.h>
#include <signal.h>


/*
 * Macros...
 */

#define writeCommand(s,n) fwrite((s), 1, (n), stdout)

#define MAX_RASTER_HORIZONTAL_SIZE	80
#define DEF_IMAGE_AREA_LEFT		1
#define MAX_IMAGE_AREA_RIGHT		81
#define MAX_HORIZONTAL_SKIP_SIZE	71
#define MAX_FEED_SIZE			255
#define RASTER_VERTICAL_SIZE		48

/*
 * Globals...
 */
 
unsigned char	*PrintBuffer;		/* Pointer to print buffer */
unsigned char	*PixelBuffer;		/* Pointer to pixel buffer */
int		Page;			/* Current page number */
int		Canceled;		/* Has the current job been canceled? */
int		PrintFlag;		/* Print flag in the page */
int		PrintBytesPerLine;	/* Number of bytes per line for fitpos filter */
int		LeftOffset;		/* Left offset */
int		NumBlankLine;		/* Number of blank line */
int		NumPrintLine;		/* Number of print line */
int		FirstPrintPixel;	/* First byte position containing print pixel in the line */
int		LastPrintPixel;		/* Last byte position containing print pixel in the line */


/*
 * Prototypes...
 */

int	GetChoiceID(ppd_file_t *ppd, char *optionName);
void	OutputPrintData(cups_page_header_t *header, unsigned char *printData);
void	OutputFeed(void);
void	OutputRaster(ppd_file_t *ppd, cups_page_header_t *header);
void	StartJob(ppd_file_t *ppd);
void	StartPage(ppd_file_t *ppd, cups_page_header_t *header);
void	EndPage(ppd_file_t *ppd, cups_page_header_t *header);
void	EndJob(ppd_file_t *ppd);
void	CancelJob(int sig);


/*
 * 'GetChoiceID()' - Get PPD choice ID.
 */

int					/* O - PPD choice ID */
GetChoiceID(ppd_file_t  *ppd,		/* I - PPD file */
            char        *optionName)	/* I - PPD option keyword name */
{
  ppd_choice_t *choice;
  ppd_option_t *option;

  choice = ppdFindMarkedChoice(ppd, optionName);

  if (choice == NULL)
  {
    option = ppdFindOption(ppd, optionName);

    if (option != NULL)
    {
      choice = ppdFindChoice(option,option->defchoice);

      if (choice == NULL)
        return -1;
    }
    else
      return -1;
  }

  return atoi(choice->choice);
}


/*
 * 'OutputPrintData()' - Output print data.
 */

void
OutputPrintData(cups_page_header_t *header,	/* I - Page header */
                unsigned char *PrintData)	/* I - Print data */
{
  int	x,y;		/* Looping vars */
  int	offset;		/* Offset to current line */
  int	pos;		/* Output potision */
  char	pixel[1];	/* Output pixel data */

  fprintf(stderr, "DEBUG: FirstPrintPixel  = %d\n", FirstPrintPixel);
  fprintf(stderr, "DEBUG: LastPrintPixel   = %d\n", LastPrintPixel);
  fprintf(stderr, "DEBUG: NumPrintLine     = %d\n", NumPrintLine);

  pixel[0]= 0x00;

  if( FirstPrintPixel < LeftOffset )
    FirstPrintPixel = LeftOffset;

  if( LastPrintPixel < FirstPrintPixel )
    LastPrintPixel = FirstPrintPixel;

  putchar(0x1d);
  putchar(0x65);
  putchar(0x76);
  putchar(FirstPrintPixel - LeftOffset);
  putchar(LastPrintPixel - FirstPrintPixel + 1);
  putchar(NumPrintLine);

  for (y = 0; y < NumPrintLine; y++)
  {
    offset = PrintBytesPerLine * y;

    for (x = FirstPrintPixel; x < LastPrintPixel + 1; x++)
    {
      pos = offset + x;

      pixel[0] |= ((PrintData[pos] & 0x01) << 7);
      pixel[0] |= ((PrintData[pos] & 0x02) << 5);
      pixel[0] |= ((PrintData[pos] & 0x04) << 3);
      pixel[0] |= ((PrintData[pos] & 0x08) << 1);
      pixel[0] |= ((PrintData[pos] & 0x10) >> 1);
      pixel[0] |= ((PrintData[pos] & 0x20) >> 3);
      pixel[0] |= ((PrintData[pos] & 0x40) >> 5);
      pixel[0] |= ((PrintData[pos] & 0x80) >> 7);

      putchar(pixel[0]);
      pixel[0] = 0x00;
    }
  }

  fflush(stdout);
  FirstPrintPixel = MAX_HORIZONTAL_SKIP_SIZE;
  LastPrintPixel = 0;
  NumPrintLine = 0;
}


/*
 * 'OutputFeed()' - Output paper feed.
 */

void
OutputFeed(void)
{
  while (NumBlankLine > MAX_FEED_SIZE)
  {
      writeCommand("\033J", 2);
      putchar(MAX_FEED_SIZE);
      NumBlankLine -= MAX_FEED_SIZE;
  }
  writeCommand("\033J", 2);
  putchar(NumBlankLine);
  NumBlankLine = 0;
}


/*
 * 'OutputRaster()' - Output raster data.
 */

void
OutputRaster(ppd_file_t         *ppd,		/* I - PPD file */
             cups_page_header_t *header)	/* I - Page header */
{
  int	i;		/* Looping vars */
  int	print_data;	/* Is there print data in the line? */
  int	choice_id;	/* PPD choice ID */

 /*
  * Check raster data
  */

  print_data = 0;

  for (i = 0; i < PrintBytesPerLine; i++)
  {
    if (PixelBuffer[i] != 0x00)
    {
      if (i < FirstPrintPixel)	FirstPrintPixel = i;
      if (i > LastPrintPixel)	LastPrintPixel = i;
      print_data = 1;
    }
  }

 /*
  * Output raster data
  */

  if (print_data)
  {
    choice_id = GetChoiceID(ppd, "fitCutPosition");

    if ((PrintFlag == 0) &&
        ((choice_id == 1)||(choice_id == 2)))
    {
      PrintFlag = 1;
      NumBlankLine = 0;
    }
    else
    {
      if (NumBlankLine > 0)
        OutputFeed();
    }

    if (PrintBuffer != NULL)
    { /* band mode */
      memcpy(PrintBuffer + (PrintBytesPerLine * NumPrintLine),
             PixelBuffer, PrintBytesPerLine);

      NumPrintLine++;

      if (NumPrintLine == RASTER_VERTICAL_SIZE)
        OutputPrintData(header, PrintBuffer);
    }
    else
    { /* line mode */
      NumPrintLine = 1;
      OutputPrintData(header, PixelBuffer);
    }
  }
  else
  {
    if (PrintBuffer != NULL)
    { /* band mode */
      if (NumPrintLine > 0)
        OutputPrintData(header, PrintBuffer);
    }

    NumBlankLine++;
  }
}


/*
 * 'StartJob()' - Initialize printer and output printer control command.
 */

void
StartJob(ppd_file_t *ppd)		/* I - PPD file */
{
  int	choice_id;			/* PPD choice ID */

 /*
  * Initialize printer
  */

  writeCommand("\033@", 2);             /* Initialize printer */
  writeCommand("\035eu\103\001", 5);    /* Set monochrome */
  writeCommand("\035P\313\313", 4);     /* Set horizontal and vertical motion units */

 /*
  * Buzzer
  */

  choice_id = GetChoiceID(ppd, "fitBuzzer");

  if ( choice_id == 1 )
  {
    /*choice_id = GetChoiceID(ppd, "fitTimeOfBeep");
    writeCommand("\033p\002", 3);
    putchar(choice_id/2);
	putchar(choice_id/2);
    */
    
    /* Buzzer Pattern & Repeat */
    choice_id = GetChoiceID(ppd, "fitBuzPattern");
    if ( choice_id == 0 )
  	{
	    writeCommand("\033p\003", 3);
	    putchar(0);
    	putchar(0);
    	putchar(0);
    }
    
    else
    {
		switch( choice_id )
		{
			case 1:
				choice_id = GetChoiceID(ppd, "fitBuzRepeat");
				writeCommand("\033p\003", 3);
			    putchar(40/2);
		    	putchar(120/2);
		    	putchar(choice_id);
		    	break;
		    	
		    case 2:
		    	choice_id = GetChoiceID(ppd, "fitBuzRepeat");
				writeCommand("\033p\003", 3);
			    putchar(140/2);
		    	putchar(140/2);
		    	putchar(choice_id);
		    	break;
		    
		    case 3:
		    	choice_id = GetChoiceID(ppd, "fitBuzRepeat");
				writeCommand("\033p\003", 3);
			    putchar(200/2);
		    	putchar(200/2);
		    	putchar(choice_id);
		    	break;
		    
		    case 4:
		    	choice_id = GetChoiceID(ppd, "fitBuzRepeat");
				writeCommand("\033p\003", 3);
			    putchar(10/2);
		    	putchar(50/2);
		    	putchar(choice_id);
		    	break;
		    	
		    case 5:
		    	choice_id = GetChoiceID(ppd, "fitBuzRepeat");
				writeCommand("\033p\003", 3);
			    putchar(30/2);
		    	putchar(50/2);
		    	putchar(choice_id);
		    	break;
		    	
		    default:break;
		}
	}
    
    
  }

 /*
  * Drawer #1
  */

  choice_id = GetChoiceID(ppd, "fitDrawer1");

  if ( choice_id == 1 )
  {
    choice_id = GetChoiceID(ppd, "fitTimeOfPulseDrawer1");
    writeCommand("\033p\000", 3);
    putchar(choice_id/2);
    putchar(choice_id/2);
  }

 /*
  * Drawer #2
  */

  choice_id = GetChoiceID(ppd, "fitDrawer2");

  if ( choice_id == 1 )
  {
    choice_id = GetChoiceID(ppd, "fitTimeOfPulseDrawer2");
    writeCommand("\033p\001", 3);
    putchar(choice_id/2);
    putchar(choice_id/2);
  }
}


/*
 * 'StartPage()' - Start a page of graphics.
 */

void
StartPage(ppd_file_t         *ppd,	/* I - PPD file */
          cups_page_header_t *header)	/* I - Page header */
{
  int		choice_id;	/* PPD choice ID */
  ppd_option_t	*option;	/* Pointer to PPD option structure */
  ppd_choice_t	*choice;	/* Pointer to PPD option choice structure */
  ppd_size_t	*size;		/* Pointer to PPD page size structure */

  fprintf(stderr, "DEBUG: fittopos StartPage ********************\n");
  fprintf(stderr, "DEBUG: MediaClass = \"%s\"\n", header->MediaClass);
  fprintf(stderr, "DEBUG: MediaColor = \"%s\"\n", header->MediaColor);
  fprintf(stderr, "DEBUG: MediaType  = \"%s\"\n", header->MediaType);
  fprintf(stderr, "DEBUG: OutputType = \"%s\"\n", header->OutputType);

  fprintf(stderr, "DEBUG: AdvanceDistance = %d\n", header->AdvanceDistance);
  fprintf(stderr, "DEBUG: AdvanceMedia  = %d\n", header->AdvanceMedia);
  fprintf(stderr, "DEBUG: Collate       = %d\n", header->Collate);
  fprintf(stderr, "DEBUG: CutMedia      = %d\n", header->CutMedia);
  fprintf(stderr, "DEBUG: Duplex        = %d\n", header->Duplex);
  fprintf(stderr, "DEBUG: HWResolution  = [ %d %d ]\n", header->HWResolution[0],
          header->HWResolution[1]);
  fprintf(stderr, "DEBUG: ImagingBoundingBox = [ %d %d %d %d ]\n",
          header->ImagingBoundingBox[0], header->ImagingBoundingBox[1],
          header->ImagingBoundingBox[2], header->ImagingBoundingBox[3]);
  fprintf(stderr, "DEBUG: InsertSheet   = %d\n", header->InsertSheet);
  fprintf(stderr, "DEBUG: Jog           = %d\n", header->Jog);
  fprintf(stderr, "DEBUG: LeadingEdge   = %d\n", header->LeadingEdge);
  fprintf(stderr, "DEBUG: Margins       = [ %d %d ]\n", header->Margins[0],
          header->Margins[1]);
  fprintf(stderr, "DEBUG: ManualFeed    = %d\n", header->ManualFeed);
  fprintf(stderr, "DEBUG: MediaPosition = %d\n", header->MediaPosition);
  fprintf(stderr, "DEBUG: MediaWeight   = %d\n", header->MediaWeight);
  fprintf(stderr, "DEBUG: MirrorPrint   = %d\n", header->MirrorPrint);
  fprintf(stderr, "DEBUG: NegativePrint = %d\n", header->NegativePrint);
  fprintf(stderr, "DEBUG: NumCopies     = %d\n", header->NumCopies);
  fprintf(stderr, "DEBUG: Orientation   = %d\n", header->Orientation);
  fprintf(stderr, "DEBUG: OutputFaceUp  = %d\n", header->OutputFaceUp);
  fprintf(stderr, "DEBUG: PageSize      = [ %d %d ]\n", header->PageSize[0],
          header->PageSize[1]);
  fprintf(stderr, "DEBUG: Separations   = %d\n", header->Separations);
  fprintf(stderr, "DEBUG: TraySwitch    = %d\n", header->TraySwitch);
  fprintf(stderr, "DEBUG: Tumble        = %d\n", header->Tumble);
  fprintf(stderr, "DEBUG: cupsWidth     = %d\n", header->cupsWidth);
  fprintf(stderr, "DEBUG: cupsHeight    = %d\n", header->cupsHeight);
  fprintf(stderr, "DEBUG: cupsMediaType = %d\n", header->cupsMediaType);
  fprintf(stderr, "DEBUG: cupsBitsPerColor = %d\n", header->cupsBitsPerColor);
  fprintf(stderr, "DEBUG: cupsBitsPerPixel = %d\n", header->cupsBitsPerPixel);
  fprintf(stderr, "DEBUG: cupsBytesPerLine = %d\n", header->cupsBytesPerLine);
  fprintf(stderr, "DEBUG: cupsColorOrder   = %d\n", header->cupsColorOrder);
  fprintf(stderr, "DEBUG: cupsColorSpace   = %d\n", header->cupsColorSpace);
  fprintf(stderr, "DEBUG: cupsCompression  = %d\n", header->cupsCompression);

 /*
  * output printer control command.
  */

  if ( Page == 1 )
  {
   /*
    * Start raster command
    */

    writeCommand("\035ey", 3);

   /*
    * Top of Form
    */

    choice_id = GetChoiceID(ppd, "fitTopOfForm");

    switch ( choice_id )
    {
      case 45 :
          writeCommand("\035ew\000", 4);
          writeCommand("\035ex\001", 4);
          break;

      case 120 :
      default :
          writeCommand("\035ex\000", 4);
          break;
    }

   /*
    * Paper Cut Start of Document
    */

    choice_id = GetChoiceID(ppd, "fitCutDoc");

    if ( choice_id == 1 )
      writeCommand("\035V\102\000", 4);	/* partial cut */
    else if ( choice_id == 2 )
      writeCommand("\035V\101\000", 4);	/* full cut */
  }
  else if ( Page > 1 )
  {
   /*
    * Paper Cut End of Page
    */

    choice_id = GetChoiceID(ppd, "fitCutPage");

    if ( choice_id == 1 )
      writeCommand("\035V\102\000", 4);	/* partial cut */
    else if ( choice_id == 2 )
      writeCommand("\035V\101\000", 4);	/* full cut */
  }

 /*
  *  Set bytes number in the line.
  */
  if (header->cupsBytesPerLine > MAX_RASTER_HORIZONTAL_SIZE)
  { /* for cups v1.1.23 */
    LeftOffset = DEF_IMAGE_AREA_LEFT;
    PrintBytesPerLine = MAX_IMAGE_AREA_RIGHT;

    choice = ppdFindMarkedChoice(ppd, "PageSize");

    if (choice == NULL)
    {
      option = ppdFindOption(ppd, "PageSize");

      if (option != NULL)
        choice = ppdFindChoice(option,option->defchoice);
    }

    if (choice != NULL)
    {
      size = ppdPageSize(ppd, choice->choice);

      fprintf(stderr, "DEBUG: PaperSize = \"%s\"\n", choice->choice);
      fprintf(stderr, "DEBUG: width  = \"%f\"\n", size->width);
      fprintf(stderr, "DEBUG: height = \"%f\"\n", size->length);
      fprintf(stderr, "DEBUG: left   = \"%f\"\n", size->left);
      fprintf(stderr, "DEBUG: bottom = \"%f\"\n", size->bottom);
      fprintf(stderr, "DEBUG: right  = \"%f\"\n", size->right);
      fprintf(stderr, "DEBUG: top    = \"%f\"\n", size->top);

      if (size != NULL)
      {
        if (size->right > 0)
        {
          LeftOffset  = (size->left / 72 * header->HWResolution[0]) / 8;

          if (header->cupsBytesPerLine > MAX_IMAGE_AREA_RIGHT)
          {
            PrintBytesPerLine = ((size->right / 72 
                                       * header->HWResolution[0]) + 7) / 8;

            if (PrintBytesPerLine > MAX_IMAGE_AREA_RIGHT)
              PrintBytesPerLine = MAX_IMAGE_AREA_RIGHT;
          }
        }
      }
    }
  }
  else
  { /* for cups v1.3 */
    LeftOffset = 0;
    PrintBytesPerLine = header->cupsBytesPerLine;
  }

  fprintf(stderr, "DEBUG: LeftOffset = \"%d\"\n", LeftOffset);
  fprintf(stderr, "DEBUG: PrintBytesPerLine = \"%d\"\n", PrintBytesPerLine);

 /*
  * Allocate memory
  */

  if ((PixelBuffer = malloc(header->cupsBytesPerLine)) != NULL)
  {
    memset(PixelBuffer, 0x00, header->cupsBytesPerLine);
  }
  else
  {
    fputs("ERROR: Unable to allocate memory!\n", stderr);
    exit(1);
  }

  if ((PrintBuffer = malloc(PrintBytesPerLine * RASTER_VERTICAL_SIZE)) != NULL)
    memset(PrintBuffer, 0x00, PrintBytesPerLine * RASTER_VERTICAL_SIZE);

  FirstPrintPixel = MAX_HORIZONTAL_SKIP_SIZE;
  LastPrintPixel = 0;
  NumPrintLine = 0;
  NumBlankLine = 0;
  PrintFlag = 0;
}


/*
 * 'EndPage()' - Finish a page of graphics.
 */

void
EndPage(ppd_file_t         *ppd,	/* I - PPD file */
        cups_page_header_t *header)	/* I - Page header */
{
  int	choice_id;			/* PPD choice ID */

 /*
  * Eject the current page...
  */

  if (PrintBuffer != NULL)
  { /* band mode */
    if (NumPrintLine > 0)
      OutputPrintData(header, PrintBuffer);
  }

 /*
  * Cut position
  */

  choice_id = GetChoiceID(ppd, "fitCutPosition");
  if ((choice_id == 0)||
      (choice_id == 2)||
      ((choice_id == 1)&&(PrintFlag == 0)))
  {
    NumBlankLine = 0;
  }

  OutputFeed();

 /*
  * Paper feed 4.5mm
  */

  writeCommand("\033J\044", 3);

  fflush(stdout);

 /*
  * Free memory...
  */

  if (PixelBuffer)
    free(PixelBuffer);

  if (PrintBuffer)
    free(PrintBuffer);
}


/*
 * 'EndJob()' - Finish a job.
 */

void
EndJob(ppd_file_t *ppd)	/* I - PPD file */
{
  int	choice_id;		/* PPD choice ID */

 /*
  * Paper Cut End of Document
  */

  choice_id = GetChoiceID(ppd, "fitCutDoc");

  if ( choice_id == 3 )
    writeCommand("\035V\102\000", 4);	/* partial cut */
  else if ( choice_id == 4 )
    writeCommand("\035V\101\000", 4);	/* full cut */

  writeCommand("\035ez", 3);            /* End raster command */
  writeCommand("\035ex\000", 4);        /* Cancel top of form */

 /*
  * Drawer #1
  */

  choice_id = GetChoiceID(ppd, "fitDrawer1");

  if ( choice_id == 2 )
  {
    choice_id = GetChoiceID(ppd, "fitTimeOfPulseDrawer1");
    writeCommand("\033p\000", 3);
    putchar(choice_id/2);
    putchar(choice_id/2);
  }

 /*
  * Drawer #2
  */

  choice_id = GetChoiceID(ppd, "fitDrawer2");
  if ( choice_id == 2 )
  {
    choice_id = GetChoiceID(ppd, "fitTimeOfPulseDrawer2");
    writeCommand("\033p\001", 3);
    putchar(choice_id/2);
    putchar(choice_id/2);
  }

 /*
  * Buzzer
  */

  choice_id = GetChoiceID(ppd, "fitBuzzer");
  if ( choice_id == 2 )
  {
    /*choice_id = GetChoiceID(ppd, "fitTimeOfBeep");
    writeCommand("\033p\002", 3);
    putchar(choice_id/2);
    putchar(choice_id/2);
    */
    
        /* Buzzer Pattern & Repeat */
    choice_id = GetChoiceID(ppd, "fitBuzPattern");
    if ( choice_id == 0 )
  	{
	    writeCommand("\033p\003", 3);
	    putchar(0);
    	putchar(0);
    	putchar(0);
    }
    
    else
    {
		switch( choice_id )
		{
			case 1:
				choice_id = GetChoiceID(ppd, "fitBuzRepeat");
				writeCommand("\033p\003", 3);
			    putchar(40/2);
		    	putchar(120/2);
		    	putchar(choice_id);
		    	break;
		    	
		    case 2:
		    	choice_id = GetChoiceID(ppd, "fitBuzRepeat");
				writeCommand("\033p\003", 3);
			    putchar(140/2);
		    	putchar(140/2);
		    	putchar(choice_id);
		    	break;
		    
		    case 3:
		    	choice_id = GetChoiceID(ppd, "fitBuzRepeat");
				writeCommand("\033p\003", 3);
			    putchar(200/2);
		    	putchar(200/2);
		    	putchar(choice_id);
		    	break;
		    
		    case 4:
		    	choice_id = GetChoiceID(ppd, "fitBuzRepeat");
				writeCommand("\033p\003", 3);
			    putchar(10/2);
		    	putchar(50/2);
		    	putchar(choice_id);
		    	break;
		    	
		    case 5:
		    	choice_id = GetChoiceID(ppd, "fitBuzRepeat");
				writeCommand("\033p\003", 3);
			    putchar(30/2);
		    	putchar(50/2);
		    	putchar(choice_id);
		    	break;
		    	
		    default:break;
		}
	}
    
    
    
  }

 /*
  * Initialize printer
  */

  writeCommand("\033@", 2);
}


/*
 * 'CancelJob()' - Cancel the current job...
 */

void
CancelJob(int sig)			/* I - Signal */
{
  (void)sig;

  Canceled = 1;
}


/*
 * 'main()' - Main entry and processing of driver.
 */

int					/* O - Exit status */
main(int  argc,				/* I - Number of command-line arguments */
     char *argv[])			/* I - Command-line arguments */
{
  int			fd;		/* File descriptor */
  cups_raster_t		*ras;		/* Raster stream for printing */
  cups_page_header_t	header;		/* Page header from file */
  ppd_file_t		*ppd;		/* PPD file */
  int			num_options;	/* Number of options */
  cups_option_t		*options;	/* Options */
  int			y;		/* Current line */
#if defined(HAVE_SIGACTION) && !defined(HAVE_SIGSET)
  struct sigaction action;		/* Actions for POSIX signals */
#endif /* HAVE_SIGACTION && !HAVE_SIGSET */


 /*
  * Make sure status messages are not buffered...
  */

  setbuf(stderr, NULL);

 /*
  * Check command-line...
  */

  if (argc < 6 || argc > 7)
  {
    fputs("ERROR: rastertofp2200pos job-id user title copies options [file]\n", stderr);
    return (1);
  }

 /*
  * Open the PPD file...
  */

  ppd = ppdOpenFile(getenv("PPD"));

  if (!ppd)
  {
    fputs("ERROR: Unable to open PPD file!\n", stderr);
    return (1);
  }

  ppdMarkDefaults(ppd);

  num_options = cupsParseOptions(argv[5], 0, &options);

  if ((num_options != 0) && (options != NULL))
  {
    cupsMarkOptions(ppd, num_options, options);
    cupsFreeOptions(num_options, options);
  }

 /*
  * Open the page stream...
  */

  if (argc == 7)
  {
    if ((fd = open(argv[6], O_RDONLY)) == -1)
    {
      perror("ERROR: Unable to open raster file - ");
      sleep(1);
      return (1);
    }
  }
  else
    fd = 0;

  ras = cupsRasterOpen(fd, CUPS_RASTER_READ);

 /*
  * Register a signal handler to eject the current page if the
  * job is cancelled.
  */

  Canceled = 0;

#ifdef HAVE_SIGSET /* Use System V signals over POSIX to avoid bugs */
  sigset(SIGTERM, CancelJob);
#elif defined(HAVE_SIGACTION)
  memset(&action, 0, sizeof(action));

  sigemptyset(&action.sa_mask);
  action.sa_handler = CancelJob;
  sigaction(SIGTERM, &action, NULL);
#else
  signal(SIGTERM, CancelJob);
#endif /* HAVE_SIGSET */

 /*
  * Initialize the print device...
  */

  StartJob(ppd);

 /*
  * Process pages as needed...
  */

  Page = 0;

  while (cupsRasterReadHeader(ras, &header))
  {
   /*
    * Write a status message with the page number and number of copies.
    */

    if (Canceled)
      break;

    Page ++;

    fprintf(stderr, "PAGE: %d %d\n", Page, header.NumCopies);

   /*
    * Start the page...
    */

    StartPage(ppd, &header);

   /*
    * Loop for each line on the page...
    */

    for (y = 0; y < header.cupsHeight; y ++)
    {
     /*
      * Let the user know how far we have progressed...
      */

      if (Canceled)
        break;

      if ((y & 127) == 0)
        fprintf(stderr, "INFO: Printing Page %d, %d%% complete...\n", Page,
                100 * y / header.cupsHeight);

     /*
      * Read a line of graphics...
      */

      if (cupsRasterReadPixels(ras, PixelBuffer, header.cupsBytesPerLine) < 1)
        break;

     /*
      * Write it to the printer...
      */

      OutputRaster(ppd, &header);
    }

   /*
    * Eject the page...
    */

    EndPage(ppd, &header);

    if (Canceled)
      break;
  }

 /*
  * Finish a job...
  */

  EndJob(ppd);

  if (ppd)
    ppdClose(ppd);

 /*
  * Close the raster stream...
  */

  cupsRasterClose(ras);

  if (fd != 0)
    close(fd);

 /*
  * If no pages were printed, send an error message...
  */

  if (Page == 0)
    fputs("ERROR: No pages found!\n", stderr);
  else
    fputs("INFO: Ready to print.\n", stderr);

  return (Page == 0);
}


/*
 * End of "rastertofp2200pos.c v102 2009-11-12 18:03:00"
 */
